<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="khtmlttsplugin" library="khtmlttsplugin" version="4" translationDomain="khtmltts">
	<MenuBar>
		<Menu name="tools">
		<text>&amp;Tools</text>
			<Action name="tools_tts"/>
		</Menu>
	</MenuBar>
	<ToolBar name="speechToolbar" hidden="true">
		<text>Speech Toolbar</text>
		<Action name="tools_tts"/>
	</ToolBar>
</gui>

