<!DOCTYPE gui>
<gui name="kshellcmdplugin" library="konq_shellcmdplugin" translationDomain="kshellcmdplugin">
<MenuBar>
 <Menu name="tools"><text>&amp;Tools</text>
  <Action name="executeshellcommand"/>
 </Menu>
</MenuBar>
</gui>
