<!DOCTYPE gui>
<gui name="FSViewPart" library="fsviewpart" version = "2" translationDomain="fsview">
<MenuBar>
 <Menu name="edit"><text>&amp;Edit</text>
  <Action name="new_menu"/>
  <Separator/>
  <Action name="rename"/>
  <Action name="move_to_trash" />
  <Action name="delete"/>
  <Action name="editMimeType"/>
  <Action name="properties"/>
 </Menu>
 <Menu name="view"><Text>&amp;View</Text>
  <Separator/>
  <Action name="treemap_visdir"/>
  <Action name="treemap_colordir"/>
  <Action name="treemap_areadir"/>
  <Action name="treemap_depthdir"/>
 </Menu>
 <Menu name="help"><text>&amp;Help</text>
  <Action name="help_fsview"/>
 </Menu>
</MenuBar>
<State name="has_selection" >
  <enable>
   <Action name="move_to_trash" />
   <Action name="delete" />
  </enable>
 </State>
 <State name="has_no_selection" >
  <disable>
   <Action name="rename" />
   <Action name="move_to_trash" />
   <Action name="delete" />
  </disable>
</State>
</gui>
