<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="searchbar" library="searchbarplugin" version="5" translationDomain="searchbarplugin">
<ToolBar fullWidth="true" name="locationToolBar" newline="true"><text>Search Toolbar</text>
  <Action name="toolbar_search_bar" />
</ToolBar>
</gui>
